/*
    Copyright (c) 2013 aizo ag, Zurich, Switzerland

    This file is part of libdsuid.

    libdsuid is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    libdsuid is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with digitalSTROM Server. If not, see <http://www.gnu.org/licenses/>.
*/

#include <check.h>
#include <uuid.h>
#include <stdbool.h>
#include <stdlib.h>

#include "dsuid.h"

START_TEST(test_dsuid_equal)
{
    uuid_rc_t rc;
    int equal;

    /*
     * @see strcmp
     * a > b -> -1
     * a < b -> 1
     * a == b -> 0
     */

    /* dsuid1 < dsuid2 */
    dsuid_t dsuid1 = { { 0x35, 0x04, 0x17, 0x5f, 0xe0, 0x01, 0x00, 0x00, 0x00, 0x00, 0x02, 0x03, 0x0a, 0x0b, 0x0c, 0x0d, 0x00 } };
    dsuid_t dsuid2 = { { 0x35, 0x04, 0x17, 0x5f, 0xe0, 0x01, 0x00, 0x00, 0x00, 0x00, 0x02, 0x03, 0x0a, 0x0b, 0xff, 0xff, 0x00 } };

    rc = uuid_compare((uuid_t *)dsuid1.id, (uuid_t *)dsuid2.id, &equal);
    ck_assert_uint_eq(rc, UUID_RC_OK);
    ck_assert_int_eq(equal, -1);

    rc = uuid_compare((uuid_t *)dsuid2.id, (uuid_t *)dsuid1.id, &equal);
    ck_assert_uint_eq(rc, UUID_RC_OK);
    ck_assert_int_eq(equal, 1);

    rc = uuid_compare((uuid_t *)dsuid1.id, (uuid_t *)dsuid1.id, &equal);
    ck_assert_uint_eq(rc, UUID_RC_OK);
    ck_assert_int_eq(equal, 0);

    ck_assert(dsuid_equal(&dsuid1, &dsuid2) ==  false);
    ck_assert(dsuid_equal(&dsuid1, &dsuid1) ==  true);
}
END_TEST

START_TEST(test_dsuid_is_null)
{
    dsuid_t dsuid = { { 0x35, 0x04, 0x17, 0x5f, 0xe0, 0x01, 0x00, 0x00, 0x00, 0x00, 0x02, 0x03, 0x0a, 0x0b, 0x0c, 0x0d, 0x00 } };

    ck_assert(dsuid_is_null(&dsuid) == false);

    memset(&dsuid, 0, sizeof(dsuid_t));
    ck_assert(dsuid_is_null(&dsuid) == true);

    memset(&dsuid, 0x7f, sizeof(dsuid_t));
    ck_assert(dsuid_is_null(&dsuid) == false);

    dsuid = DSUID_NULL;
    ck_assert(dsuid_is_null(&dsuid) == true);

    dsuid = DSUID_BROADCAST;
    ck_assert(!dsuid_is_null(&dsuid));
}
END_TEST

START_TEST(test_dsuid_is_broadcast)
{
    dsuid_t dsuid;

    memset(&dsuid, 0xff, sizeof(dsuid_t));
    ck_assert(dsuid_is_broadcast(&dsuid));

    memset(&dsuid, 0x7f, sizeof(dsuid_t));
    ck_assert(!dsuid_is_broadcast(&dsuid));

    dsuid = DSUID_BROADCAST;
    ck_assert(dsuid_is_broadcast(&dsuid));

    dsuid = DSUID_NULL;
    ck_assert(!dsuid_is_broadcast(&dsuid));
}
END_TEST

START_TEST(test_dsuid_unaligned)
{
    dsuid_t *dsuid;
    char *buf;

    /*
     * INTEL supports unalinged memory access
     * ARM enable alignment warning, then check
     * the system log for warnings:
     *  echo 3 > /proc/cpu/alignment
     *  http://www.mjmwired.net/kernel/Documentation/arm/mem_alignment
     */

    buf = malloc(DSUID_SIZE + 1);
    ck_assert_ptr_ne(buf, NULL);
    dsuid = (dsuid_t *)(buf + 1);

    memset(dsuid, 0xff, sizeof(*dsuid));
    ck_assert(dsuid_is_broadcast(dsuid));
    ck_assert(dsuid_equal(dsuid, &DSUID_BROADCAST));

    memset(dsuid, 0x0, sizeof(*dsuid));
    ck_assert(dsuid_is_null(dsuid));
    ck_assert(dsuid_equal(dsuid, &DSUID_NULL));

    free(buf);
}
END_TEST

Suite *dsuid_suite()
{
    Suite *s = suite_create("dsuid");
    TCase *tc_dsuid = tcase_create("dsuid");

    tcase_set_timeout(tc_dsuid, 4);

    tcase_add_test(tc_dsuid, test_dsuid_is_null);
    tcase_add_test(tc_dsuid, test_dsuid_equal);
    tcase_add_test(tc_dsuid, test_dsuid_is_broadcast);
    tcase_add_test(tc_dsuid, test_dsuid_unaligned);
    suite_add_tcase(s, tc_dsuid);
    return s;
}

int main(int argc, char **argv)
{
    (void)argc;
    (void)argv;

    Suite *dsuid = dsuid_suite();
    SRunner *test_runner = srunner_create(dsuid);
    srunner_run_all(test_runner, CK_NORMAL);
    int failed = 0;
    failed = srunner_ntests_failed(test_runner);
    srunner_free(test_runner);
    return failed;
}
